@@ -4,19 +4,20 @@ from datetime import datetime
 
 from logger_tt import logger
 
-from metaDMG.fit import fit_utils, serial
+from metaDMG.fit.serial import run_single_config
+from metaDMG.utils import Configs
 
 
-def run_workflow(configs: fit_utils.Configs):
+def run_workflow(configs: Configs):
 
     parallel_samples = min(configs["parallel_samples"], len(configs))
 
     logger.info(f"Running metaDMG on {len(configs)} files in total.")
 
     if parallel_samples == 1 or len(configs) == 1:
-        logger.info(f"Running in seriel (1 core)")
+        logger.info(f"Running in serial (1 core)")
         for config in configs:
-            dfs = serial.run_single_config(config)
+            dfs = run_single_config(config)
             # df_mismatches, df_fit_results, df_results = dfs
 
     else:
@@ -25,6 +26,6 @@ def run_workflow(configs: fit_utils.Configs):
 
         with Pool(max_workers=parallel_samples) as pool:
             # for dfs in pool.imap_unordered(serial.run_single_config, configs):
-            for dfs in pool.map(serial.run_single_config, configs):
+            for dfs in pool.map(run_single_config, configs):
                 pass
                 # df_mismatches, df_fit_results, df_results = dfs
